@@ -1,7 +1,7 @@
 /*
  *  configfile.c
  *
- *  Copyright (C) 1999, 2001, 2006, 2007 Staf Wagemakers Belgie/Belgium
+ *  Copyright (C) 1999, 2001, 2006, 2007, 2020 Staf Wagemakers Belgie/Belgium
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
  */
 
 #include "configfile.h"
+#include <string.h>
 #define BUFFER_LEN 200
 /*
  * converts line to a string with only the global part
@@ -55,7 +56,7 @@ while(1) {
       strcat(end_section,*current_section);
       strcat(end_section,">");
   
-      if((found=istrstr(s,end_section))==NULL) {
+      if((found=strcasestr(s,end_section))==NULL) {
 	 break;
       }
       s=found+strlen(end_section);
@@ -88,7 +89,6 @@ if(fp==NULL) return NULL;
 if(mode!=2) fseek(fp,0,SEEK_SET);
   else var_name[0]='\0';
 
-if (s==NULL) return(NULL);
 if (section_name!=NULL) {
    start_section=(char *)xmalloc(strlen(section_name)+1+strlen("<")+strlen(">"));
    end_section=(char *)xmalloc(strlen(section_name)+1+strlen("</")+strlen(">"));
@@ -109,13 +109,13 @@ while (fgets(s,BUFFER_LEN,fp)) {
   if(ss==NULL) return (NULL);
   if (section_name!=NULL) {
      if (!sect) {
-        if((c=istrstr(ss,start_section))!=NULL) {
+        if((c=strcasestr(ss,start_section))!=NULL) {
           ss=c;sect=1;
         }
      }
  
    if (sect) {
-      if ((c=istrstr(ss,end_section))!=NULL) {
+      if ((c=strcasestr(ss,end_section))!=NULL) {
          ss=c;sect=0;
       }
     }
@@ -140,10 +140,10 @@ while (fgets(s,BUFFER_LEN,fp)) {
   }
 }
 
-if((mode==1)&&(ss!=NULL)) free(ss); 
-if(start_section!=NULL) free(start_section); 
-if(end_section!=NULL) free(end_section);
-if(current_section!=NULL) free(current_section);
+if((mode==1)&&(ss!=NULL)) xfree(ss); 
+if(start_section!=NULL) xfree(start_section); 
+if(end_section!=NULL) xfree(end_section);
+if(current_section!=NULL) xfree(current_section);
 return(r);
 }
 
@@ -299,11 +299,11 @@ return(real_get_config_array(fp,NULL,var_name,1));
  * 
  * parms[0][0] = variable name , parms[0][1] = value , ...
  */
-char * add_parms(char *txt,char *parms[][2]) {
+char * add_parms(char *txt,char ***parms) {
 char *ret,*c,*txt2;
 c=txt2=replace_headtail_needles(txt,parms,"%{","}");
 ret=cut_between(txt2,"%{","}");
-free(c);
+xfree(c);
 return(ret);
 }
 /* ------------------------------------------- */
